/// <reference types="node" />
import { Server, ServerRequest, Agent } from "http";
import { EventEmitter } from "events";
import * as pathToRegexp from "path-to-regexp";
/**
 * 代理规则
 */
export interface Rule {
    /** 规则，可为字符串或正则表达式 */
    match: string | RegExp;
    /** 生成目标URL的规则 */
    proxy: string | ProxyHandler;
    /** 额外增加的请求头，仅当proxy为string类型时有效 */
    headers?: HttpHeaders;
}
/**
 * 生成目标URL的函数
 *
 * @param req ServerRequest请求对象
 * @param result 匹配到的URL参数
 */
export declare type ProxyHandler = (req: ServerRequest, result?: RegExpExecArray) => ProxyResult;
/**
 * Debug函数
 */
export declare type DebugHandler = (...args: any[]) => void;
/**
 * 获取Agent的函数
 */
export declare type GetAgentHandler = () => Agent | false | undefined;
/**
 * HTTP请求头
 */
export declare type HttpHeaders = Record<string, string | string[] | undefined>;
/**
 * 已格式化处理的代理规则
 */
export interface FormattedRule {
    /** 规则正则表达式 */
    match: RegExp;
    keys: pathToRegexp.Key[];
    /** 生成目标URL的函数 */
    proxy: ProxyHandler;
    /** 额外增加的请求头 */
    headers: HttpHeaders;
    /** 规则的字符串ID */
    id: string;
}
/**
 * 代理目标结果
 */
export interface ProxyResult {
    /** 目标URL */
    url: string;
    /** 额外增加的请求头 */
    headers: HttpHeaders;
}
/**
 * 查找匹配路由规则的结果
 */
export interface FindRuleResult {
    rule: FormattedRule;
    result: RegExpExecArray;
}
/**
 * HTTP代理类
 */
export default class HTTPProxy extends EventEmitter {
    private static _counter;
    /** Server对象 */
    private readonly _server;
    /** 代理规则 */
    private readonly _rules;
    /** 调试函数 */
    private _debug;
    /** 生成Agent的函数 */
    private _getAgent;
    /** 请求计数器 */
    private _httpProxyCounter;
    constructor();
    /**
     * 处理HTTP请求
     *
     * @param req
     * @param res
     */
    private _onRequest(req, res);
    /**
     * 处理CONNECT请求
     *
     * @param req
     * @param socket
     * @param bodyHead
     */
    private _onConnect(req, socket, bodyHead);
    /**
     * 向客户端响应出错信息
     *
     * @param res
     * @param status
     * @param msg
     */
    private _responseError(res, status?, msg?);
    /**
     * 向客户端响应本地文件
     *
     * @param res
     * @param file
     * @param headers
     */
    private _responseLocalFile(res, file, headers);
    /**
     * 向客户端响应欢迎页面
     *
     * @param res
     */
    private _responseWelcomePage(res);
    /**
     * 根据URL匹配第一个符合的规则
     *
     * @param url
     */
    private _findRuleByUrl(url);
    /**
     * HTTP代理转发
     *
     * @param req
     * @param res
     * @param options
     */
    private _httpProxyPass(req, res, options?);
    /**
     * 格式化代理规则
     *
     * @param rule
     */
    private _formatRule(rule);
    /**
     * 编译代理规则字符串
     *
     * @param match
     * @param keys
     * @param url
     */
    private _compileProxyString(match, keys, url, headers?);
    /**
     * 增加代理规则
     *
     * @param rule 规则
     */
    addRule(rule: Rule): void;
    /**
     * 删除代理规则
     *
     * @param rule 规则
     */
    removeRule(rule: Rule): void;
    /**
     * 删除所有代理规则
     */
    removeAllRules(): void;
    /**
     * Server对象
     */
    readonly server: Server;
    /**
     * 设置debug函数
     */
    debugHandler: DebugHandler;
    /**
     * 设置生成Agent函数
     */
    agentHandler: GetAgentHandler;
}
