"use strict";
/**
 * @leizm/http-proxy
 *
 * @author Zongmin Lei <leizongmin@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
const net_1 = require("net");
const fs = require("fs");
const path = require("path");
const http_1 = require("http");
const https_1 = require("https");
const events_1 = require("events");
const url_1 = require("url");
const pathToRegexp = require("path-to-regexp");
const mime = require("mime");
const createDebug = require("debug");
/**
 * 从URL中获取域名和端口
 *
 * @param url
 */
function getHostPortFromUrl(url) {
    const info = url_1.parse(url);
    const defaultPort = isHttpsProtocol(info.protocol) ? 443 : 80;
    return { host: info.hostname || "", port: Number(info.port || defaultPort) };
}
/**
 * 检查是否为https协议
 *
 * @param protocol 协议名
 */
function isHttpsProtocol(protocol = "http:") {
    protocol = protocol || "http:";
    return protocol.toLowerCase() === "https:";
}
/**
 * 判断不为URL
 */
function isNotUrl(url) {
    url = url.toLowerCase();
    return !(url.indexOf("http://") === 0 || url.indexOf("https://") === 0);
}
/**
 * 删除URL中的查询字符串
 *
 * @param url
 */
function removeUrlQueryString(url) {
    return splitUrlQueryString(url).base;
}
/**
 * 分开URL的基本部分和查询字符串部分
 *
 * @param url
 */
function splitUrlQueryString(url) {
    const i = url.indexOf("?");
    if (i === -1) {
        return { base: url, qs: "" };
    }
    return { base: url.slice(0, i), qs: url.slice(i) };
}
/**
 * HTTP代理类
 */
class HTTPProxy extends events_1.EventEmitter {
    constructor() {
        super();
        /** 代理规则 */
        this._rules = new Map();
        /** 调试函数 */
        this._debug = createDebug(`http-proxy:#${HTTPProxy._counter++}`);
        /** 生成Agent的函数 */
        this._getAgent = () => false;
        /** 请求计数器 */
        this._httpProxyCounter = 0;
        this._server = http_1.createServer();
        // 处理普通请求
        this._server.on("request", this._onRequest.bind(this));
        // HTTPS请求只能直接转发
        this._server.on("connect", this._onConnect.bind(this));
        // 处理出错信息
        this._server.on("error", err => {
            this._debug("server error: %s", err);
            this.emit("error", err);
        });
        this.on("proxy", info => this._debug("proxy: %j", info));
        this._debug("inited");
    }
    /**
     * 处理HTTP请求
     *
     * @param req
     * @param res
     */
    _onRequest(req, res) {
        this._debug("on request: %s %s %j", req.method, req.url, req.headers);
        if (!req.url) {
            return this._responseError(res, 500, "invalid request");
        }
        const { base, qs } = splitUrlQueryString(req.url);
        const ret = this._findRuleByUrl(base);
        if (ret) {
            this._debug("http proxy pass by rule: %j", ret);
            const result = ret.rule.proxy(req, ret.result);
            result.url += qs;
            this._httpProxyPass(req, res, result);
        }
        else {
            this._httpProxyPass(req, res);
        }
    }
    /**
     * 处理CONNECT请求
     *
     * @param req
     * @param socket
     * @param bodyHead
     */
    _onConnect(req, socket, bodyHead) {
        this._debug("on connect: %s %s", req.method, req.url);
        const url = `https://${req.url || ""}`;
        const { host, port } = getHostPortFromUrl(url);
        const remoteSocket = new net_1.Socket();
        this._debug("connecting to: %s:%s", host, port);
        this.emit("proxy", { origin: req.url, target: url, method: req.method, rewrite: false });
        remoteSocket.connect(port, host, () => {
            const content = `HTTP/${req.httpVersion} 200 Connection established\r\n\r\n`;
            remoteSocket.write(bodyHead);
            socket.write(content);
        });
        remoteSocket.pipe(socket);
        remoteSocket.on("error", err => {
            this._debug("remote socket error: %s", err);
            const content = `HTTP/${req.httpVersion} 500 Connection error\r\n\r\n`;
            socket.end(content);
        });
        socket.pipe(remoteSocket);
        socket.on("error", err => {
            this._debug("source socket on error: %s", err);
            remoteSocket.end();
        });
    }
    /**
     * 向客户端响应出错信息
     *
     * @param res
     * @param status
     * @param msg
     */
    _responseError(res, status = 500, msg = "internal error") {
        this._debug("response error: %s %s", status, msg);
        this.emit("responseError", status, msg);
        res.writeHead(status, {
            "content-type": "text/html",
        });
        fs.readFile(path.resolve(__dirname, "../files/error.html"), (err, tpl) => {
            if (err) {
                this.emit("error", err);
                res.end(`<h1>HTTP ${status} <small>${msg}</small></h1>`);
                return;
            }
            const html = tpl
                .toString()
                .replace(/\{\{title\}\}/g, "服务器出错")
                .replace(/\{\{message\}\}/g, `HTTP ${status} ${msg}`);
            res.end(html);
        });
    }
    /**
     * 向客户端响应本地文件
     *
     * @param res
     * @param file
     * @param headers
     */
    _responseLocalFile(res, file, headers) {
        this._debug("response local file: %s", file);
        file = removeUrlQueryString(file);
        const type = mime.getType(file) || "text/plain";
        fs.stat(file, (err, stats) => {
            if (err) {
                return this._responseError(res, err.code === "ENOENT" ? 404 : 403, err.message);
            }
            if (stats.isDirectory()) {
                return this._responseLocalFile(res, path.join(file, "index.html"), headers);
            }
            const s = fs.createReadStream(file);
            s.on("error", err => {
                this._responseError(res, 500, err.message);
            });
            s.on("open", () => {
                res.writeHead(200, Object.assign({}, headers, { "content-type": type, "content-length": stats.size }));
                s.pipe(res);
            });
        });
    }
    /**
     * 向客户端响应欢迎页面
     *
     * @param res
     */
    _responseWelcomePage(res) {
        this._debug("[#%s] welcome page");
        this._responseLocalFile(res, path.resolve(__dirname, "../files/welcome.html"), {});
    }
    /**
     * 根据URL匹配第一个符合的规则
     *
     * @param url
     */
    _findRuleByUrl(url) {
        const keys = this._rules.keys();
        for (const key of keys) {
            this._debug("_findRuleByUrl: url=%s, key=%s", url, key);
            const result = key.exec(url);
            if (result) {
                return { rule: this._rules.get(key), result };
            }
        }
        return;
    }
    /**
     * HTTP代理转发
     *
     * @param req
     * @param res
     * @param options
     */
    _httpProxyPass(req, res, options) {
        const url = (options ? options.url : req.url) || "";
        const headers = options ? options.headers : {};
        const num = ++this._httpProxyCounter;
        const info = url_1.parse(url);
        // 如果是直接打开代理服务器，显示欢迎页面
        if (!url_1.parse(req.url || "").hostname) {
            return this._responseWelcomePage(res);
        }
        this._debug("[#%s] http proxy pass: %s %j", num, url, headers);
        this.emit("proxy", { origin: req.url, target: url, method: req.method, rewrite: req.url !== url });
        if (isNotUrl(url)) {
            // 本地文件代理
            this._debug("[#%s] proxy local file: %s", url);
            this._responseLocalFile(res, url, headers);
        }
        else {
            // 处理connection请求头
            if (req.headers["proxy-connection"]) {
                headers["connection"] = req.headers["proxy-connection"];
                delete req.headers["proxy-connection"];
            }
            // HTTP代理
            const request = isHttpsProtocol(info.protocol) ? https_1.request : http_1.request;
            const remoteReq = request({
                host: info.hostname,
                port: info.port ? Number(info.port) : 80,
                method: req.method,
                path: info.path,
                headers: Object.assign({}, req.headers, headers),
                agent: this._getAgent(),
            }, remoteRes => {
                this._debug("[#%s] remote response: %s %j", num, remoteRes.statusCode, remoteRes.headers);
                res.writeHead(remoteRes.statusCode || 200, remoteRes.headers);
                remoteRes.pipe(res);
            });
            remoteReq.on("error", err => {
                this._debug("[#%s] remote request error: %s", num, err);
                this._responseError(res, 500, err.stack);
            });
            req.on("error", err => {
                this._debug("[#%s] source request error: %s", num, err);
                this._responseError(res, 500, err.stack);
            });
            req.on("close", () => {
                this._debug("[#%s] source request close", num);
            });
            req.pipe(remoteReq);
        }
    }
    /**
     * 格式化代理规则
     *
     * @param rule
     */
    _formatRule(rule) {
        const keys = [];
        const match = pathToRegexp(rule.match, keys, { end: true });
        const proxy = typeof rule.proxy === "function" ? rule.proxy : this._compileProxyString(match, keys, rule.proxy, rule.headers);
        const headers = rule.headers || {};
        return { match, keys, id: String(match), proxy, headers };
    }
    /**
     * 编译代理规则字符串
     *
     * @param match
     * @param keys
     * @param url
     */
    _compileProxyString(match, keys, url, headers = {}) {
        const { hostname } = url_1.parse(url);
        const handler = (req, result) => {
            const ret = {
                url,
                headers: {},
            };
            if (hostname) {
                ret.headers["host"] = hostname;
            }
            if (result) {
                keys.forEach((k, i) => {
                    ret.url = ret.url.replace(`{${k.name}}`, result[i]);
                });
                result.forEach((v, i) => {
                    ret.url = ret.url.replace(`{${i}}`, v);
                });
            }
            ret.headers = Object.assign({}, ret.headers, headers);
            this._debug("reset target url: %s => %s", req.url, ret.url);
            return ret;
        };
        return handler;
    }
    /**
     * 增加代理规则
     *
     * @param rule 规则
     */
    addRule(rule) {
        const r = this._formatRule(rule);
        this._debug("add rule: %j", r);
        this._rules.set(r.match, r);
        this.emit("addRule", rule);
    }
    /**
     * 删除代理规则
     *
     * @param rule 规则
     */
    removeRule(rule) {
        const r = this._formatRule(rule);
        this._debug("remove rule: %j", r);
        this._rules.forEach((item, key) => {
            if (item.id === r.id) {
                this._rules.delete(key);
            }
        });
        this.emit("removeRule", rule);
    }
    /**
     * 删除所有代理规则
     */
    removeAllRules() {
        this._debug("remove all rules: %j", this._rules);
        this._rules.clear();
    }
    /**
     * Server对象
     */
    get server() {
        return this._server;
    }
    /**
     * 设置debug函数
     */
    set debugHandler(fn) {
        this._debug = fn;
    }
    /**
     * 设置生成Agent函数
     */
    set agentHandler(fn) {
        this._getAgent = fn;
    }
}
HTTPProxy._counter = 0;
exports.default = HTTPProxy;
//# sourceMappingURL=index.js.map